{

  Creator:  Kevin Morris
  Build Date:   2020-Aug-08

  This Lazarus project was built to demonstrate how Lazarus can connect to
  the RingCentral REST server and objtain a "Token".

  I am donating this code to the Public Domain.  I am submitting this to the
  RingCentral forum so that other RingCentral / Lazarus users can use it to learn from.

}






unit f_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ValEdit, ComCtrls, ExtCtrls, IdHTTP, IdSSLOpenSSL, IdLogDebug, IdIntercept,
  IdGlobal, fpjson, jsonparser;

type

  { TfrmMain }

  TfrmMain = class(TForm)
    bbtnGetToken: TButton;
    Bevel1: TBevel;
    btnClose: TButton;
    hnd: TIdSSLIOHandlerSocketOpenSSL;   // Check the .lfm for specific non-default settings required
    htp: TIdHTTP;  // Check the .lfm for specific non-default settings required
    log: TIdLogDebug;  // Check the .lfm for specific non-default settings required
    memSndRcv: TMemo;
    memFmtJson1: TMemo;
    Panel1: TPanel;
    pgcMain: TPageControl;
    tabRawRsp: TTabSheet;
    tabKVP: TTabSheet;
    tabFmtJson: TTabSheet;
    vle: TValueListEditor;
    xbxJSON: TCheckBox;
    procedure bbtnGetTokenClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure logReceive(ASender: TIdConnectionIntercept; var ABuffer: TIdBytes);
    procedure logSend(ASender: TIdConnectionIntercept; var ABuffer: TIdBytes);
    procedure LoadDefaults;
    procedure LoadPersistent;
  private
    sent_lst: TStringList;
    rcvd_lst: TStringList;
    procedure ConvertJSONToKeyVals;
  public

  end;

var
  frmMain: TfrmMain;

implementation

{$R *.lfm}

{ TfrmMain }

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  LoadDefaults;
  LoadPersistent;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  // Since these are .Create -ed without an owner, they have to be manually .Free d
  // They are initialized with nil, so we know that the .Free will work, even if their .Create failed
  sent_lst.Free;
  rcvd_lst.Free;
end;

procedure TfrmMain.logReceive(ASender: TIdConnectionIntercept; var ABuffer: TIdBytes);
var i: Integer; s: String;
begin
  s := '';
  for i := Low(ABuffer) to High(ABuffer) do
    s := s + Chr(ABuffer[i]);
  rcvd_lst.Add(s);  // We use TStringList to collect all the lines together, then we'll add them to the TMemo as a single consecutive block
end;

procedure TfrmMain.logSend(ASender: TIdConnectionIntercept; var ABuffer: TIdBytes);
var i: Integer; s: String;
begin
  s := '';
  for i := Low(ABuffer) to High(ABuffer) do
    s := s + Chr(ABuffer[i]);
  sent_lst.Add(s);  // We use TStringList to collect all the lines together, then we'll add them to the TMemo as a single consecutive block
end;

procedure TfrmMain.LoadDefaults;
begin
  // Configure the default settings
  pgcMain.ActivePageIndex := 0;

  // Initialize to nil, in case something goes wrong during .Create
  sent_lst := nil;
  rcvd_lst := nil;

  // Instead of using the "Position" settings, which does most of this automatically,
  // I chose to increase the height and position manually bc the final presentation is better (simpler layout)

  Top := 0;
  Left := (Screen.WorkAreaWidth - Width) div 2;
  Height := Screen.WorkAreaHeight - 40;

end;

procedure TfrmMain.LoadPersistent;
begin
  // Load values from ini file on hard drive
  //  -- none --

  //
  sent_lst := TStringList.Create;
  rcvd_lst := TStringList.Create;
end;

procedure TfrmMain.bbtnGetTokenClick(Sender: TObject);   // Reference:  https://developers.ringcentral.com/guide/authentication/password-flow
var prms_lst: TStringList;
begin
  memSndRcv.Lines.Clear;
  memSndRcv.Lines.Add(' ');
  memFmtJson1.Lines.Clear;

  vle.Clear;

  sent_lst.Clear;
  rcvd_lst.Clear;

  prms_lst := nil;
  try
    prms_lst := TStringList.Create;

    Screen.Cursor := crHourGlass;
    Application.ProcessMessages;

    {

    Obtaining the values you need from RingCentral for the HTTP Request are a major source of confusion.
    Pay particular attention to the nomenclature.

    Be sure you have created a "Sandbox" Account and then create a "non-UI" app with Password Flow as the authentication method.
    Then go into the "Sandbox Credentials", and enter the values ... name-for-name ... from that credentials box.

    }


    htp.Request.Username := '<your own Client ID>';       // Client ID
    htp.Request.Password := '<your own Client Secret'';   //  Client Secret

    prms_lst.Add('grant_type=password');
    prms_lst.Add('username=<your own account>');      // Account
    prms_lst.Add('password=<your own account password'); // Password  (This is the password for the Account)

    htp.Post('https://platform.devtest.ringcentral.com/restapi/oauth/token', prms_lst);   // Remove the text ".devtest" for the production version of your code

    memSndRcv.Lines.Add('Sent:');
    memSndRcv.Lines.Add(' ');
    memSndRcv.Lines.AddStrings(sent_lst);
    memSndRcv.Lines.Add(' ');
    memSndRcv.Lines.Add(' ');
    memSndRcv.Lines.Add('Rcvd:');
    memSndRcv.Lines.Add(' ');
    memSndRcv.Lines.AddStrings(rcvd_lst);
    memSndRcv.Lines.Add(' ');

  finally
    prms_lst.Free;

    if xbxJSON.Checked then
      begin
        ConvertJSONToKeyVals;
      end;

    Screen.Cursor := crDefault;
    Application.ProcessMessages;
  end;



end;

procedure TfrmMain.ConvertJSONToKeyVals;     // Reference:  https://wiki.freepascal.org/fcl-json
var
  jData : TJSONData;
  jObject : TJSONObject;
  s : String;

  i: Integer;  is_json: Boolean;
begin
  // We received a bunch of text in the rcvd_lst, only some of which is JSON, and the JSON may be in pieces.
  // Let's reassemble here to get one block of text that we can subsequently step through to pick out the value(s) we need

  i := Pos('{', rcvd_lst.Text);
  s := Trim(Copy(rcvd_lst.Text, i, Pred(Length(rcvd_lst.Text) - i)));
  while Pos(#13#10, s) > 0 do
    s := StringReplace(s, #13#10, '', [rfReplaceAll]);
  memFmtJson1.Text := s;

  // Even though carriage return + line feed combos (#13#10) are eliminated, the line feeds (#10s alone) are retained,
  // so we end up with a nice looking block of JSON text, that can now be converted into a TJSONObject

  // Ensure the "access_token" tag exists.  If not, bail out now.
  if Pos('access_token', s) = 0 then
    begin
      ShowMessage('Hey, weird... there is no access_token" tag.' + #13#10 + 'Unable to get the token from the response text');
      Exit;
    end;

  try
    // Create from string
    jData := GetJSON(memFmtJson1.Text);

    // Cast as TJSONObject to make access easier
    jObject := TJSONObject(jData);

    // Retrieve token
    s := jObject.Get('access_token');
    vle.InsertRow('access_token', s, True);

    // Retrieve the other values
    s := jObject.Get('token_type');
    vle.InsertRow('token_type', s, True);
    s := jObject.Get('expires_in');
    vle.InsertRow('expires_in', s, True);
    s := jObject.Get('refresh_token');
    vle.InsertRow('refresh_token', s, True);
    s := jObject.Get('refresh_token_expires_in');
    vle.InsertRow('refresh_token_expires_in', s, True);
    s := jObject.Get('scope');
    vle.InsertRow('scope', s, True);
    s := jObject.Get('owner_id');
    vle.InsertRow('owner_id', s, True);
    s := jObject.Get('endpoint_id');
    vle.InsertRow('endpoint_id', s, True);

    vle.AutoSizeColumns;
  finally
    jObject.Free;
  end;
end;


end.

